/***************************************************************************
Copyright (c) 2007 Cory Burgett, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_REGEX_H
#define SMART_TOOLS_REGEX_H

#include <pcrecpp.h>
#include <string.h>
#include "smarttools.h"
#include "ststring.h"

class StRegExp
{
protected:
	StString regex;
	pcrecpp::RE *re;
	
public:
	
	/*! Construct an StRegEx object using a Perl regex.
		\param regex the regex to compile
	*/
	StRegExp(StString regex)
	{
		this->regex = regex;
		re = new pcrecpp::RE((std::string)regex);
	}
	
	~StRegExp() { delete re; }
	
	/*! This function does a simple test to see whether or not a certain string
		matches a regex.
		\param text the string to test against a stored regex.
		\return true if text matches the regex, false otherwise.
	*/
	bool match(StString text);
	
	/*! Resets the stored regex as if being reconstructed.
		\param regex the new regex
	*/
	void reset(StString regex)
	{
		delete re;
		this->regex = regex;
		re = new pcrecpp::RE((std::string)regex);
	}
	
	StString toStString() const { return regex; }
	
	operator StString() const { return (std::string)regex; }
};

/*! \class StRegExp
	The StRegExp class provides a means for C++ programs using SmartTools to
	store and manipulate Perl-style regular expressions (regexes). It is based on top of
	the PCRE library and can accept any Perl5 regex.
	
	Here's an example of usage using match() which tests different ways you can say "hello":
	
	\code
	StRegExp hello("h?[ae]ll?o+");
	
	if (hello.match("hello"))
		stout << "hello matches" << endl;
	if (hello.match("hallo"))
		stout << "hallo matches" << endl;
	if (hello.match("elo"))
		stout << "elo matches" << endl;
	if (hello.match("hellooo"))
		stout << "hellooo matches" << endl;
	else
		stout << "Something went seriously wrong." << endl;
	\endcode
	
	You can construct an StRegExp with any Perl-compatible regular expression.
	So, read up about Perl regexes to learn more precisely how to use
	regexes.
*/

#endif
